/*
 * GOATTRACKER "console" output routines
 */

#define MAX_COLUMNS 80
#define MAX_ROWS 30

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "bme.h"

int initscreen(void);
void closescreen(void);
void fliptoscreen(void);
void printtext(int x, int y, int color, const char *text);
void printtextc(int y, int color, const char *text);
void printblank(int x, int y, int length);
void printblankc(int x, int y, int color, int length);
void printbg(int x, int y, int color, int length);
void drawbox(int x, int y, int color, int sx, int sy);

unsigned char *scrbuffer;
unsigned char font[4096];
unsigned char fgcolor2tbl[] = {0, 0, 0, 0, 0, 0, 0, 8, 0, 1, 2, 7, 7, 7, 6, 7};

int initscreen(void)
{
  int handle;

  if (!gfx_init(320, 240, 60, GFX_SCANLINES)) return 0;
  gfx_loadpalette("goattrk.pal");
  gfx_setpalette();

  handle = io_open("font.bin");
  if (handle == -1) return 0;
  io_read(handle, font, 2048);
  io_close(handle);

  scrbuffer = malloc(MAX_COLUMNS * MAX_ROWS * 2);
  if (!scrbuffer) return 0;

  return 1;
}

void closescreen(void)
{
  if (scrbuffer)
  {
    free(scrbuffer);
    scrbuffer = NULL;
  }
}

void clearscreen(void)
{
  int c;
  char *dptr = scrbuffer;

  for (c = 0; c < MAX_ROWS * MAX_COLUMNS; c++)
  {
    *dptr++ = 0x20;
    *dptr++ = 0x7;
  }
}

void printtext(int x, int y, int color, const char *text)
{
  char *dptr = scrbuffer + ((x + y * MAX_COLUMNS) << 1);

  while (*text)
  {
    *dptr++ = *text++;
    *dptr++ = color;
  }
}

void printblank(int x, int y, int length)
{
  char *dptr = scrbuffer + ((x + y * MAX_COLUMNS) << 1);

  while (length--)
  {
    *dptr++ = 32;
    *dptr++ = 0;
  }
}

void printblankc(int x, int y, int color, int length)
{
  char *dptr = scrbuffer + ((x + y * MAX_COLUMNS) << 1);

  while (length--)
  {
    *dptr++ = 32;
    *dptr++ = color;
  }
}

void drawbox(int x, int y, int color, int sx, int sy)
{
  char *dptr = scrbuffer + ((x + y * MAX_COLUMNS) << 1);
  char *dptr2 = scrbuffer + ((x + (y+sy-1) * MAX_COLUMNS) << 1);
  int counter = sx;

  while (counter--)
  {
    *dptr++ = 205;
    *dptr++ = color;
    *dptr2++ = 205;
    *dptr2++ = color;
  }

  dptr = scrbuffer + ((x + y * MAX_COLUMNS) << 1);
  dptr2 = scrbuffer + (((x+sx-1) + y * MAX_COLUMNS) << 1);
  counter = sy;

  while (counter--)
  {
    dptr[0] = 186;
    dptr[1] = color;
    dptr2[0] = 186;
    dptr2[1] = color;
    dptr += (MAX_COLUMNS << 1);
    dptr2 += (MAX_COLUMNS << 1);
  }
  scrbuffer[(x + y * MAX_COLUMNS) << 1] = 201;
  scrbuffer[((x+sx-1) + y * MAX_COLUMNS) << 1] = 187;
  scrbuffer[(x + (y+sy-1) * MAX_COLUMNS) << 1] = 200;
  scrbuffer[((x+sx-1) + (y+sy-1) * MAX_COLUMNS) << 1] = 188;
}

void printbg(int x, int y, int color, int length)
{
  char *dptr = scrbuffer + ((x + y * MAX_COLUMNS) << 1);

  while (length--)
  {
    dptr++;
    *dptr = 15 | (color << 4);
    dptr++;
  }
}

void printtextc(int y, int color, const char *text)
{
  int x = (80 - strlen(text)) / 2;
  char *dptr = scrbuffer + ((x + y * MAX_COLUMNS) << 1);

  while (*text)
  {
    *dptr++ = *text++;
    *dptr++ = color;
  }
}

void fliptoscreen(void)
{
  unsigned char *scrptr = scrbuffer;
  int x,y,c;

  for (y = 0; y < MAX_ROWS; y++)
  {
    for (x = 0; x < MAX_COLUMNS; x++)
    {
      int bgcolor = scrptr[1] >> 4;
      int fgcolor = scrptr[1] & 15;
      int fgcolor2 = fgcolor2tbl[fgcolor];
      int charnum = scrptr[0];
      char *src = &font[charnum*8];
      char *dest = gfx_vscreen + 320 * 8 * y + x * 4;

      if (fgcolor == 7) fgcolor2 = 8;

      for (c = 0; c < 8; c++)
      {
        int d;
        char byte = *src++;

        switch (byte & 3)
        {
          case 0:
          dest[3] = bgcolor;
          break;

          case 2:
          dest[3] = fgcolor2;
          break;

          case 3:
          dest[3] = fgcolor;
          break;
        }
        byte >>= 2;
        switch (byte & 3)
        {
          case 0:
          dest[2] = bgcolor;
          break;

          case 2:
          dest[2] = fgcolor2;
          break;

          case 3:
          dest[2] = fgcolor;
          break;
        }
        byte >>= 2;
        switch (byte & 3)
        {
          case 0:
          dest[1] = bgcolor;
          break;

          case 2:
          dest[1] = fgcolor2;
          break;

          case 3:
          dest[1] = fgcolor;
          break;
        }
        byte >>= 2;
        switch (byte & 3)
        {
          case 0:
          dest[0] = bgcolor;
          break;

          case 2:
          dest[0] = fgcolor2;
          break;

          case 3:
          dest[0] = fgcolor;
          break;
        }

        dest += 320;
      }
      scrptr += 2;
    }
  }
  gfx_updatepage();
}


