/*
 * GOATTRACKER "console" output routines
 */

#define MAX_COLUMNS 80
#define MAX_ROWS 30

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "bme.h"

int initscreen(void);
void closescreen(void);
void fliptoscreen(void);
void printtext(int x, int y, int color, const char *text);
void printtextc(int y, int color, const char *text);
void printblank(int x, int y, int length);
void printblankc(int x, int y, int color, int length);
void printbg(int x, int y, int color, int length);
void drawbox(int x, int y, int color, int sx, int sy);

unsigned char *scrbuffer;

unsigned char font[4096];

int initscreen(void)
{
  int handle;

  if (!gfx_init(640, 480, 60, 0)) return 0;
  gfx_loadpalette("goattrk.pal");
  gfx_setpalette();

  handle = io_open("font.bin");
  if (handle == -1) return 0;
  io_read(handle, font, 4096);
  io_close(handle);

  scrbuffer = malloc(MAX_COLUMNS * MAX_ROWS * 2);
  if (!scrbuffer) return 0;

  return 1;
}

void closescreen(void)
{
  if (scrbuffer)
  {
    free(scrbuffer);
    scrbuffer = NULL;
  }
}

void clearscreen(void)
{
  int c;
  char *dptr = scrbuffer;

  for (c = 0; c < MAX_ROWS * MAX_COLUMNS; c++)
  {
    *dptr++ = 0x20;
    *dptr++ = 0x7;
  }
}

void printtext(int x, int y, int color, const char *text)
{
  char *dptr = scrbuffer + ((x + y * MAX_COLUMNS) << 1);

  while (*text)
  {
    *dptr++ = *text++;
    *dptr++ = color;
  }
}

void printblank(int x, int y, int length)
{
  char *dptr = scrbuffer + ((x + y * MAX_COLUMNS) << 1);

  while (length--)
  {
    *dptr++ = 32;
    *dptr++ = 0;
  }
}

void printblankc(int x, int y, int color, int length)
{
  char *dptr = scrbuffer + ((x + y * MAX_COLUMNS) << 1);

  while (length--)
  {
    *dptr++ = 32;
    *dptr++ = color;
  }
}

void drawbox(int x, int y, int color, int sx, int sy)
{
  char *dptr = scrbuffer + ((x + y * MAX_COLUMNS) << 1);
  char *dptr2 = scrbuffer + ((x + (y+sy-1) * MAX_COLUMNS) << 1);
  int counter = sx;

  while (counter--)
  {
    *dptr++ = 205;
    *dptr++ = color;
    *dptr2++ = 205;
    *dptr2++ = color;
  }

  dptr = scrbuffer + ((x + y * MAX_COLUMNS) << 1);
  dptr2 = scrbuffer + (((x+sx-1) + y * MAX_COLUMNS) << 1);
  counter = sy;

  while (counter--)
  {
    dptr[0] = 186;
    dptr[1] = color;
    dptr2[0] = 186;
    dptr2[1] = color;
    dptr += (MAX_COLUMNS << 1);
    dptr2 += (MAX_COLUMNS << 1);
  }
  scrbuffer[(x + y * MAX_COLUMNS) << 1] = 201;
  scrbuffer[((x+sx-1) + y * MAX_COLUMNS) << 1] = 187;
  scrbuffer[(x + (y+sy-1) * MAX_COLUMNS) << 1] = 200;
  scrbuffer[((x+sx-1) + (y+sy-1) * MAX_COLUMNS) << 1] = 188;
}

void printbg(int x, int y, int color, int length)
{
  char *dptr = scrbuffer + ((x + y * MAX_COLUMNS) << 1);

  while (length--)
  {
    dptr++;
    *dptr = 15 | (color << 4);
    dptr++;
  }
}

void printtextc(int y, int color, const char *text)
{
  int x = (80 - strlen(text)) / 2;
  char *dptr = scrbuffer + ((x + y * MAX_COLUMNS) << 1);

  while (*text)
  {
    *dptr++ = *text++;
    *dptr++ = color;
  }
}

void fliptoscreen(void)
{
  unsigned char *scrptr = scrbuffer;
  int x,y,c;

  for (y = 0; y < MAX_ROWS; y++)
  {
    for (x = 0; x < MAX_COLUMNS; x++)
    {
      int bgcolor = scrptr[1] >> 4;
      int fgcolor = scrptr[1] & 15;
      int charnum = scrptr[0];
      char *src = &font[charnum*16];
      char *dest = gfx_vscreen + 640 * 16 * y + x * 8;
      for (c = 0; c < 16; c++)
      {
        char byte = *src++;

        if (byte & 128) dest[0] = fgcolor;
        else dest[0] = bgcolor;
        if (byte & 64) dest[1] = fgcolor;
        else dest[1] = bgcolor;
        if (byte & 32) dest[2] = fgcolor;
        else dest[2] = bgcolor;
        if (byte & 16) dest[3] = fgcolor;
        else dest[3] = bgcolor;
        if (byte & 8) dest[4] = fgcolor;
        else dest[4] = bgcolor;
        if (byte & 4) dest[5] = fgcolor;
        else dest[5] = bgcolor;
        if (byte & 2) dest[6] = fgcolor;
        else dest[6] = bgcolor;
        if (byte & 1) dest[7] = fgcolor;
        else dest[7] = bgcolor;

        dest += 640;
      }
      scrptr += 2;
    }
  }
  gfx_updatepage();
}


